#include <Servo.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_MPU6050.h>
#include <Wire.h>
#include <SD.h>

#define WRITE_TO_CARD

// some macros are used instead of static variables to save on memory 

Servo roll_servo;
#define roll_pin 10
Servo pitch_servo;
#define pitch_pin 9

#define pause_pin 5

Adafruit_MPU6050 mpu;

float axes[3] = {0.0, 0.0, 0.0};
float last_axes[3] = {0.0, 0.0, 0.0};

unsigned long last_time = 0;

#define filename_bytes 12

#define CS 6

char filename[filename_bytes];

Sd2Card card;


void setup() {
 
  Serial.begin(9600);
  Serial.println("sta rted");

  pinMode(pause_pin, INPUT);
  pinMode(CS, OUTPUT);
 
  start_mpu();
  delay(100);  
  setup_servos();
  delay(100);
  init_card(CS);
  delay(100);
  last_time = millis();  
}

void loop() {
  float elapsed = 0;
  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);
  float buf[3] = {a.gyro.x, a.gyro.y, a.gyro.z}; // roll, yaw, pitch
  
  if (not(pause_signal())) {
    unsigned long now = millis();

    elapsed = (now - last_time)/100.0;
    for (int i = 0; i <3; i++){
      axes[i] += (buf[i] - last_axes[i]) * elapsed;
    }

    pitch_servo.write(pitch_servo.read() + (axes[2] * 2.0));
    
    /*if (axes[1] > 0.1){
      int temp = roll_servo.read();
      roll_servo.write(temp + 2);
    } else if (axes[1] < -0.1){
      int temp = roll_servo.read();
      roll_servo.write(temp - 2);
    }
    */
    roll_servo.write(roll_servo.read() + (axes[1] * 3.0));
    
    last_time = now;

    for (int i = 0; i <3; i++) {
      last_axes[i] = buf[i];
    }
  }
  else {
    // this is the branch to take if the gimbal is locked
    
    roll_servo.write(90);
    pitch_servo.write(90);
    
    // where it is pointing is the origin...
    for (int i = 0; i <3; i++){
      axes[i] = 0;
      last_axes[i] = buf[i];
    }
    last_time = millis();
  }

  float to_write[4] = {elapsed, axes[0], axes[1], axes[2]};

  format_array(to_write);

  delay(5);
  
}

void format_array(float data[]) {
  char *temp_str = malloc(8);
  char *output = malloc(96);
  
  for (int i = 0; i < 4; i++){
    dtostrf(data[i], 4, 2, temp_str);
    strcat(output, temp_str);
    strcat(output, ",");
  }

  write_to_card(output);
  Serial.println(output);
  free(temp_str);
  free(output);
}

bool pause_signal() {
  /*
   * checks if the arduino needs to pause. This simplifies stuff later down the line
   * 
   */

   return digitalRead(pause_pin);
}

void setup_servos() {
  roll_servo.attach(roll_pin);
  pitch_servo.attach(pitch_pin);

  roll_servo.write(90);
  pitch_servo.write(90);
  
}

void start_mpu() {

   
  // need to use wire for the mpu because its all a bit sussy
  Wire.begin();

  // lets the mpu restart if something goes wrong. REALLY not ideal
  Wire.setTimeout(10000);

  
   // idk why mpu.begin needs these
  uint8_t addr = 0x68;
  int32_t id = 1;

  // only carry on if th mpu can be started
  
  Serial.println(mpu.begin(addr, &Wire, id));
  Serial.println("pog");
  
  Serial.println("started MPU");
  mpu.setAccelerometerRange(MPU6050_RANGE_8_G);
  mpu.setGyroRange(MPU6050_RANGE_500_DEG);
  mpu.setFilterBandwidth(MPU6050_BAND_21_HZ);
  
}

void init_card(int pin) {
  /*
   * natalie's responsibility
   * 
   * tries to intialise the SD card and get the filename to use
   */ 

  Serial.println("initialising SD card");

  
  Serial.println(SD.begin(pin));
  
  // can the card be initialised?
  if (!card.init(SPI_HALF_SPEED, CS)) {
     Serial.println("couldn't initialise SD card, exiting");
     while (true) {
      delay(1);
     }
  }
  get_filename();


  Serial.print("activating the SD card worked, filename of ");
  Serial.println(filename);

  //write_to_card("begin");
}

void write_to_card(char* to_write) {
   /*
   * natalie's responsibility
   * 
   * takes in a filename and writes to it, and closes the file afterwards 
   */
  File location = SD.open(filename, FILE_WRITE);
  location.println(to_write);
  location.close();
}

void get_filename() {
  /*
   * function to get the filename to use, so each test gets a unique 
   * file
   * 
   */
  int i = 0;

  // files will follow the pattern test(number).csv
  char base_name[] = "2a";
  char ending[] = ".txt";

  
  while (true) {
    // this checks every loop to see if a filename is free. 
    // if so, it returns it
    // if not, it tries the next one
    
    char test_filename[filename_bytes];
    sprintf(test_filename, "%s%d%s", base_name, i, ending); // formats the filename
    if (SD.exists(test_filename)) {

      i++;
    }else{
      strcpy(filename, test_filename);
      break;
    }
  }
}
